@@ -17,7 +17,7 @@ using namespace apolloron;
 
 const char *INKF_PROGNAME = "inkf";
 const char *INKF_BRIEF    = "Yet Another Network Kanji Filter";
-const char *INKF_VERSION  = "2.0.1";
+const char *INKF_VERSION  = "2.0.3";
 const char *INKF_COPY     = "Copyright (C) 2002-2018 Masashi Astro Tachibana";
 const char *INKF_DEF_OUT  = "inkf.out";
 
@@ -57,6 +57,17 @@ typedef struct {
     bool flag_hiragana;
     bool flag_katakana;
     bool flag_midi;
+    bool flag_md5;
+    bool flag_sha1;
+    bool flag_sort_csv;
+    bool flag_sort_csv_r;
+    const char *csv_column;
+    bool flag_format_json;
+    bool flag_minify_json;
+    bool flag_re_match;
+    const char *re_match_pattern;
+    bool flag_no_return_param;
+    bool flag_no_return;
     char input_charset[32];
     char output_charset[32];
     TMIME mime_decode;
@@ -69,7 +80,9 @@ typedef struct {
 
 bool set_option(TOption &option, int argc, const char **argv);
 void guess(const String &str, const char *input_charset, FILE *fpout);
-void convert(String &str, const TOption &option, FILE *fpout);
+void convert(String &str, TOption &option, FILE *fpout);
+void set_input_charset_by_env(char *input_charset);
+void set_output_charset(char *output_charset, const char *input_charset, String &str);
 void show_help();
 void show_version();
 void show_copy();
@@ -119,11 +132,16 @@ int main(int argc, char *argv[]) {
         // read from stdin
         if (option.mime_decode == MIME_NONE && option.mime_encode == MIME_NONE &&
                 !option.flag_guess && !option.flag_html_to_plain && !option.flag_midi &&
-                strcasecmp(option.output_charset, "UTF-8_BOM") != 0 &&
+                !option.flag_md5 && !option.flag_sha1 &&
+                !option.flag_sort_csv && !option.flag_sort_csv_r &&
+                !option.flag_format_json && !option.flag_minify_json &&
+                !option.flag_re_match &&
+                strncasecmp(option.input_charset, "AUTODETECT", 10) != 0 &&
                 strncasecmp(option.input_charset, "UTF-16", 6) != 0 &&
                 strncasecmp(option.input_charset, "UTF16", 5) != 0 &&
                 strncasecmp(option.input_charset, "UTF-32", 6) != 0 &&
                 strncasecmp(option.input_charset, "UTF32", 5) != 0 &&
+                strcasecmp(option.output_charset, "UTF-8_BOM") != 0 &&
                 !(!strncasecmp(option.output_charset, "UTF-16", 6) ||
                 !strncasecmp(option.output_charset, "UTF16", 5) ||
                 !strncasecmp(option.output_charset, "UTF-32", 6) ||
@@ -259,6 +277,17 @@ bool set_option(TOption &option, int argc, const char **argv) {
     option.flag_hiragana = false;
     option.flag_katakana = false;
     option.flag_midi = false;
+    option.flag_md5 = false;
+    option.flag_sha1 = false;
+    option.flag_sort_csv = false;
+    option.flag_sort_csv_r = false;
+    option.csv_column = NULL;
+    option.flag_format_json = false;
+    option.flag_minify_json = false;
+    option.flag_re_match = false;
+    option.re_match_pattern = NULL;
+    option.flag_no_return_param = false;
+    option.flag_no_return = false;
     strcpy(option.input_charset, "AUTODETECT");
     option.output_charset[0] = '\0';
     option.mime_decode = MIME_NONE;
@@ -303,6 +332,32 @@ bool set_option(TOption &option, int argc, const char **argv) {
                 option.flag_katakana = true;
             } else if (!strcasecmp(argv[i], "--midi")) {
                 option.flag_midi = true;
+            } else if (!strcasecmp(argv[i], "--md5")) {
+                option.flag_md5 = true;
+            } else if (!strcasecmp(argv[i], "--sha1")) {
+                option.flag_sha1 = true;
+            } else if (!strncasecmp(argv[i], "--sort-csv=", 11)) {
+                option.flag_sort_csv = true;
+                option.csv_column = &(argv[i][11]);
+                if (option.csv_column[0] == '\0') {
+                    return false; // invalid parameter
+                }
+            } else if (!strncasecmp(argv[i], "--sort-csv-r=", 13)) {
+                option.flag_sort_csv_r = true;
+                option.csv_column = &(argv[i][13]);
+                if (option.csv_column[0] == '\0') {
+                    return false; // invalid parameter
+                }
+            } else if (!strcasecmp(argv[i], "--format-json")) {
+                option.flag_format_json = true;
+            } else if (!strcasecmp(argv[i], "--minify-json")) {
+                option.flag_minify_json = true;
+            } else if (!strncasecmp(argv[i], "--re-match=", 11)) {
+                option.flag_re_match = true;
+                option.re_match_pattern = &(argv[i][11]);
+                if (option.re_match_pattern[0] == '\0') {
+                    return false; // invalid parameter
+                }
             } else {
                 return false; // invalid parameter
             }
@@ -478,6 +533,12 @@ bool set_option(TOption &option, int argc, const char **argv) {
                         } else if (argv[i][j + 1] == 'Q') {
                             option.mime_encode = MIME_QUOTED_PRINTABLE;
                             j++;
+                        } else if (argv[i][j + 1] == 'b') {
+                            option.mime_encode = MIME_BASE64_FULL;
+                            j++;
+                        } else if (argv[i][j + 1] == 'q') {
+                            option.mime_encode = MIME_QUOTED_PRINTABLE_FULL;
+                            j++;
                         }
                         break;
                     case 'Z':
@@ -499,6 +560,12 @@ bool set_option(TOption &option, int argc, const char **argv) {
                             j++;
                         }
                         break;
+                    case 'N':
+                        option.flag_no_return_param = true;
+                        break;
+                    case 'n':
+                        option.flag_no_return = true;
+                        break;
                     default:
                         return false; // invalid parameter
                         break;
@@ -547,7 +614,11 @@ void guess(const String &str, const char *input_charset, FILE *fpout) {
     char endian = 'L';
 
     if (input_charset != NULL && !strncasecmp(input_charset, "AUTODETECT", 10)) {
-        charcode = str.detectCharSet();
+        if (strcasestr(input_charset, "JP")) {
+            charcode = str.detectCharSetJP();
+        } else {
+            charcode = str.detectCharSet();
+        }
 
         // for compatibility with nkf 2.x
         if (!strcmp(charcode, "CP932")) {
@@ -643,8 +714,13 @@ void guess(const String &str, const char *input_charset, FILE *fpout) {
 }
 
 
-void convert(String &str, const TOption &option, FILE *fpout) {
-    String u8str;
+void convert(String &str, TOption &option, FILE *fpout) {
+    String u8str, csv_column, re_pattern;
+    Sheet csv_sheet;
+    char tmp_input_charset[32];
+    char tmp_output_charset[32];
+    bool append_return = false;
+    char r[4];
 
     if (option.mime_decode == MIME_BASE64) {
         str = str.decodeBASE64();
@@ -652,11 +728,14 @@ void convert(String &str, const TOption &option, FILE *fpout) {
         str = str.decodeQuotedPrintable();
     } else if (option.mime_decode == MIME_AUTODETECT) {
         if (0 <= str.search("=?")) {
+            set_input_charset_by_env(option.input_charset);
             if (!strncasecmp(option.input_charset, "AUTODETECT", 10)) {
-                str = str.decodeMIME(option.input_charset, "");
+                str = str.decodeMIME(option.input_charset, "UTF-8");
             } else {
-                str = str.decodeMIME(option.input_charset, option.input_charset);
+                str = str.decodeMIME(option.input_charset, "UTF-8");
             }
+            strcpy(option.input_charset, "UTF-8");
+            append_return = true;
         } else if (str[0] != '=') {
             str = str.decodeBASE64();
         } else {
@@ -689,18 +768,120 @@ void convert(String &str, const TOption &option, FILE *fpout) {
         u8str = str.strconv(option.output_charset, "UTF-8");
         u8str = u8str.changeReturnCode(RETURN_STR[option.line_mode]);
         str = u8str.strconv("UTF-8", option.output_charset);
+        u8str.clear();
     }
+    str.gc();
     if (option.mime_encode == MIME_BASE64) {
         str = str.encodeBASE64();
     } else if (option.mime_encode == MIME_QUOTED_PRINTABLE) {
         str = str.encodeQuotedPrintable();
     } else if (option.mime_encode == MIME_BASE64_FULL) {
+        u8str = str.strconv(option.output_charset, "UTF-8");
+        if (0 < u8str.len() && option.flag_no_return_param == false && u8str.search("\n") == u8str.searchR("\n") && u8str[u8str.len()-1] == '\n'){
+            str = u8str.changeReturnCode("").strconv("UTF-8", option.output_charset);
+        }
+        u8str.clear();
+        sprintf(r, "%s\t", RETURN_STR[option.line_mode]);
         str = str.encodeMIME(option.output_charset,
-                option.output_charset, 72, RETURN_STR[option.line_mode], 'B');
+                option.output_charset, 72, r, 'B');
+        append_return = true;
     } else if (option.mime_encode == MIME_QUOTED_PRINTABLE_FULL) {
+        u8str = str.strconv(option.output_charset, "UTF-8");
+        if (0 < u8str.len() && option.flag_no_return_param ==false && u8str.search("\n") == u8str.searchR("\n") && u8str[u8str.len()-1] == '\n'){
+            str = u8str.changeReturnCode("").strconv("UTF-8", option.output_charset);
+        }
+        u8str.clear();
+        sprintf(r, "%s\t", RETURN_STR[option.line_mode]);
         str = str.encodeMIME(option.output_charset,
-                option.output_charset, 72, RETURN_STR[option.line_mode], 'Q');
+                option.output_charset, 72, r, 'Q');
+        append_return = true;
+    } else if (option.flag_sort_csv) {
+        csv_sheet.setCSV(str, option.output_charset, "UTF-8");
+        set_input_charset_by_env(tmp_input_charset);
+        if (2 <= strlen(option.csv_column) && option.csv_column[0] == '"' &&
+                option.csv_column[strlen(option.csv_column)-1] == '"') {
+            csv_column = option.csv_column;
+            csv_column = csv_column.mid(1, csv_column.len()-2).unescapeQuote(tmp_input_charset, "UTF-8");
+        } else {
+            csv_column = option.csv_column;
+            csv_column = csv_column.strconv(tmp_input_charset, "UTF-8");
+        }
+        csv_sheet.sort(csv_column.c_str());
+        strcpy(tmp_output_charset, option.output_charset);
+        set_output_charset(tmp_output_charset, option.input_charset, str);
+        str = csv_sheet.getCSV("UTF-8", tmp_output_charset,
+                RETURN_STR[option.line_mode]);
+    } else if (option.flag_sort_csv_r) {
+        csv_sheet.setCSV(str, option.output_charset, "UTF-8");
+        set_input_charset_by_env(tmp_input_charset);
+        if (2 <= strlen(option.csv_column) && option.csv_column[0] == '"' &&
+                option.csv_column[strlen(option.csv_column)-1] == '"') {
+            csv_column = option.csv_column;
+            csv_column = csv_column.mid(1, csv_column.len()-2).unescapeQuote(tmp_input_charset, "UTF-8");
+        } else {
+            csv_column = option.csv_column;
+            csv_column = csv_column.strconv(tmp_input_charset, "UTF-8");
+        }
+        csv_sheet.sortr(csv_column.c_str());
+        strcpy(tmp_output_charset, option.output_charset);
+        set_output_charset(tmp_output_charset, option.input_charset, str);
+        str = csv_sheet.getCSV("UTF-8", tmp_output_charset,
+                RETURN_STR[option.line_mode]);
+    } else if (option.flag_format_json) {
+        JSONValue json_root;
+        JSONReader reader;
+        JSONStyledWriter writer;
+        u8str = str.strconv(option.output_charset, "UTF-8");
+        reader.parse(u8str, json_root);
+        u8str.clear();
+        strcpy(tmp_output_charset, option.output_charset);
+        set_output_charset(tmp_output_charset, option.input_charset, str);
+        str = writer.write(json_root).strconv("UTF-8", tmp_output_charset);
+    } else if (option.flag_minify_json) {
+        JSONValue json_root;
+        JSONReader reader;
+        JSONFastWriter writer;
+        u8str = str.strconv(option.output_charset, "UTF-8");
+        reader.parse(u8str, json_root);
+        u8str.clear();
+        strcpy(tmp_output_charset, option.output_charset);
+        set_output_charset(tmp_output_charset, option.input_charset, str);
+        str = writer.write(json_root).strconv("UTF-8", tmp_output_charset);
+    } else if (option.flag_re_match) {
+        strcpy(tmp_output_charset, option.output_charset);
+        set_output_charset(tmp_output_charset, option.input_charset, str);
+        str = str.strconv(option.output_charset, "UTF-8");
+        set_input_charset_by_env(tmp_input_charset);
+        if (2 <= strlen(option.re_match_pattern) && option.re_match_pattern[0] == '"' &&
+                option.re_match_pattern[strlen(option.re_match_pattern)-1] == '"') {
+            re_pattern = option.re_match_pattern;
+            re_pattern = re_pattern.mid(1, re_pattern.len()-2).unescapeQuote(tmp_input_charset, "UTF-8");
+        } else {
+            re_pattern = option.re_match_pattern;
+            re_pattern = re_pattern.strconv(tmp_input_charset, "UTF-8");
+        }
+        str = str.reMatch(re_pattern.c_str());
+        str = str.strconv("UTF-8", tmp_output_charset);
     }
+    if (option.flag_md5) {
+        str = str.md5();
+        append_return = true;
+    } else if (option.flag_sha1) {
+        str = str.sha1();
+        append_return = true;
+    }
+
+    if (append_return && option.flag_no_return == false) {
+        if (str.isBinary()) {
+            u8str = str.strconv(option.output_charset, "UTF-8");
+            u8str += '\n';
+            str = u8str.strconv("UTF-8", option.output_charset);
+            u8str.clear();
+        } else {
+            str.add('\n');
+        }
+    }
+    str.gc();
 
     if (str.isBinary()) {
         fwrite(str.c_str(), 1, str.binaryLength(), fpout);
@@ -710,6 +891,40 @@ void convert(String &str, const TOption &option, FILE *fpout) {
 }
 
 
+void set_input_charset_by_env(char *input_charset) {
+    const char *env_lang;
+    env_lang = getenv("LANG");
+    if (env_lang != NULL) {
+        if (strcasestr(env_lang, "UTF-8") != NULL) {
+            strcpy(input_charset, "UTF-8");
+        } else if (strcasestr(env_lang, "eucJP") != NULL) {
+            strcpy(input_charset, "EUC-JP");
+        } else if (strcasestr(env_lang, "SJIS") != NULL ||
+                !strcasecmp(env_lang, "ja_JP.PCK")) {
+            strcpy(input_charset, "SHIFT_JIS");
+        } else if (strcasestr(env_lang, "ja") != NULL) {
+            strcpy(input_charset, "AUTODETECT_JP");
+        } else {
+            strcpy(input_charset, "AUTODETECT");
+        }
+    } else {
+        strcpy(input_charset, "AUTODETECT");
+    }
+}
+
+
+void set_output_charset(char *output_charset, const char *input_charset, String &str) {
+    if (output_charset[0] == '\0') {
+        strcpy(output_charset, input_charset);
+        if (!strcasecmp(output_charset, "AUTODETECT_JP")) {
+            strcpy(output_charset, str.detectCharSetJP());
+        } else if (!strncasecmp(output_charset, "AUTODETECT", 10)) {
+            strcpy(output_charset, str.detectCharSet());
+        }
+    }
+}
+
+
 void show_help() {
     fprintf(stderr, 
       "Usage:  %s -[flags] [--] [in file/in URL] .. [out file for -O flag]\n"
@@ -723,7 +938,7 @@ void show_help() {
       " W8             UTF-8\n"
       " W16/W32[BL]    UTF-16, UTF-32\n"
       " m[BQN0]        MIME decode [B:base64,Q:quoted,N:non-strict,0:no decode]\n"
-      " M[BQ]          MIME encode [B:base64 Q:quoted]\n"
+      " M[BQbq]        MIME encode [B:base64 Q:quoted b:base64 header q:quoted header]\n"
       " Z              Convert JISX0208 Alphabet to ASCII\n"
       " O              Output to File (DEFAULT '%s')\n"
       " L[uwm]         Line mode u:LF w:CRLF m:CR (DEFAULT noconversion)\n"
@@ -757,6 +972,13 @@ void show_help() {
       " --html-to-plain        Convert HTML to Plain Text\n"
       " -g --guess     Guess the input code\n"
       " --midi         Create MIDI object from MML like music sequencial text\n"
+      " --md5          Calc MD5 sum\n"
+      " --sha1         Calc SHA-1 sum\n"
+      " --sort-csv=<column>    Sort CSV\n"
+      " --sort-csv-r=<column>  Sort CSV (reverse)\n"
+      " --format-json  Reformat JSON\n"
+      " --minify-json  Minify JSON\n"
+      " --re-match=<pattern>   Regular Expression match\n"
       " --overwrite    Overwrite original listed files by filtered result\n"
       " -v --version   Print the version\n"
       " --help/-V      Print this help / configuration\n"
