@@ -52,7 +52,7 @@ Keys::Keys(const Keys &keys) {
     (*this).emptyString = new String;
     *((*this).emptyString) = ""; // stay empty
 
-    set(keys);
+    (*this).set(keys);
 }
 
 
@@ -61,19 +61,10 @@ Keys::Keys(const Keys &keys) {
     @return void
  */
 Keys::~Keys() {
-    (void)clear();
-}
-
-/*! Delete instance of Keys.
-    @param void
-    @retval true   success
-    @retval false  failure
- */
-bool Keys::clear() {
     int i;
     long l;
 
-    if (pElement != NULL && *pElement != NULL) {
+    if ((*this).pElement != NULL && *((*this).pElement) != NULL) {
         for (i = 0; i < 64; i++) {
             l = 0;
             while (l < (*this).nElementTotal[i]) {
@@ -97,10 +88,53 @@ bool Keys::clear() {
         delete (*this).tmpString;
         (*this).tmpString = (String *)NULL;
     }
+
     if ((*this).emptyString != (String *)NULL) {
         delete (*this).emptyString;
         (*this).emptyString = (String *)NULL;
     }
+}
+
+/*! Delete instance of Keys.
+    @param void
+    @retval true   success
+    @retval false  failure
+ */
+bool Keys::clear() {
+    int i;
+    long l;
+
+    if ((*this).pElement != NULL && *((*this).pElement) != NULL) {
+        for (i = 0; i < 64; i++) {
+            l = 0;
+            while (l < (*this).nElementTotal[i]) {
+                if ((*this).pElement[i][l] != NULL) {
+                    if ((*this).pElement[i][l]->key != NULL) {
+                        delete [] (*this).pElement[i][l]->key;
+                    }
+                    delete (*this).pElement[i][l]->value;
+                    delete [] (*this).pElement[i][l];
+                }
+                l++;
+            }
+            delete *((*this).pElement[i]);
+            (*this).nElementTotal[i] = 0;
+        }
+        delete [] *((*this).pElement);
+        (*this).pElement = NULL;
+    }
+
+    (*this).pElement = new Element ** [64];
+    for (i = 0; i < 64; i++) {
+        (*this).nElementTotal[i] = 0;
+        (*this).nElementStart[i] = -1;
+        (*this).pElement[i] = (Element **)NULL;
+    }
+
+    if ((*this).tmpString != (String *)NULL) {
+        delete (*this).tmpString;
+        (*this).tmpString = (String *)NULL;
+    }
 
     return true;
 }
