@@ -551,7 +551,7 @@ int test5() {
 
     // Set Values
     str_a = "12345/ABCDEF";                            // Set String
-    str_b = str_a.reMatch("2.*/A");                    // Get Sub-String
+    str_b = str_a.reMatch("^.*(2.*/A).*$");            // Get Sub-String
     str_c = str_a.reSubst("2.*C", "_test_");           // Replace Sub-String
     str_d = str_a.reTrans("234", "bcd");               // Replace Sub-String
 
